﻿using System;
using System.Windows.Controls;
using System.Windows.Data;
using System.Globalization;

namespace TestProject
{
    /// <summary>
    /// 입력 데이터 검증 규칙
    /// </summary>
    public class EntryDataValidationRule : ValidationRule
    {
        //////////////////////////////////////////////////////////////////////////////////////////////////// Method
        ////////////////////////////////////////////////////////////////////////////////////////// Public

        #region 검증하기 - Validate(value, cultureInfo)

        /// <summary>
        /// 검증하기
        /// </summary>
        /// <param name="value">값</param>
        /// <param name="cultureInfo">문화 정보</param>
        /// <returns>검증 결과</returns>
        public override ValidationResult Validate(object value, CultureInfo cultureInfo)
        {
            BindingGroup bindingGroup = value as BindingGroup;

            PurchaseItem item = bindingGroup.Items[0] as PurchaseItem;
            
            object priceObject;
            object offerExpireDateObject;

            bool priceResult = bindingGroup.TryGetValue(item, "Price"          , out priceObject          );
            bool dateResult  = bindingGroup.TryGetValue(item, "OfferExpireDate", out offerExpireDateObject);

            if(!priceResult || !dateResult)
            {
                return new ValidationResult(false, "속성을 찾을 수가 없습니다.");
            }

            double   price           = (double)priceObject;
            DateTime offerExpireDate = (DateTime)offerExpireDateObject;

            if(price > 100)
            {
                if(offerExpireDate < DateTime.Today + new TimeSpan(7, 0, 0, 0))
                {
                    return new ValidationResult(false, "100불 이상의 항목은 최소 7일 이내에서 이용 가능합니다.");
                }
            }

            return ValidationResult.ValidResult;
        }

        #endregion
    }
}