﻿using System;
using System.Windows;
using System.Windows.Controls;

namespace TestProject
{
    /// <summary>
    /// 메인 윈도우
    /// </summary>
    public partial class MainWnidow : Window
    {
        //////////////////////////////////////////////////////////////////////////////////////////////////// Constructor
        ////////////////////////////////////////////////////////////////////////////////////////// Public

        #region 생성자 - MainWindow()

        /// <summary>
        /// 생성자
        /// </summary>
        public MainWnidow()
        {
            InitializeComponent();

            this.stackPanel.Loaded += stackPanel_Loaded;

            this.stackPanel.AddHandler
            (
                Validation.ErrorEvent,
                new EventHandler<ValidationErrorEventArgs>(stackPanel_ValidationError)
            );

            this.submitButton.Click += submitButton_Click;
            this.cancelButton.Click += cancelButton_Click;
        }

        #endregion

        //////////////////////////////////////////////////////////////////////////////////////////////////// Constructor
        ////////////////////////////////////////////////////////////////////////////////////////// Public

        #region 스택 패널 로드시 처리하기 - stackPanel_Loaded(sender, e)

        /// <summary>
        /// 스택 패널 로드시 처리하기
        /// </summary>
        /// <param name="sender">이벤트 발생자</param>
        /// <param name="e">이벤트 인자</param>
        private void stackPanel_Loaded(object sender, RoutedEventArgs e)
        {
            this.stackPanel.DataContext = new PurchaseItem();

            this.stackPanel.BindingGroup.BeginEdit();
        }

        #endregion
        #region 스택 패널 검증 에러시 처리하기 - stackPanel_ValidationError(sender, e)

        /// <summary>
        /// 스택 패널 검증 에러시 처리하기
        /// </summary>
        /// <param name="sender">이벤트 발생자</param>
        /// <param name="e">이벤트 인자</param>
        private void stackPanel_ValidationError(object sender, ValidationErrorEventArgs e)
        {
            if(e.Action == ValidationErrorEventAction.Added)
            {
                MessageBox.Show(e.Error.ErrorContent.ToString());
            }
        }

        #endregion
        #region 제출 버튼 클릭시 처리하기 - submitButton_Click(sender, e)

        /// <summary>
        /// 제출 버튼 클릭시 처리하기
        /// </summary>
        /// <param name="sender">이벤트 발생자</param>
        /// <param name="e">이벤트 인자</param>
        private void submitButton_Click(object sender, RoutedEventArgs e)
        {
            if(this.stackPanel.BindingGroup.CommitEdit())
            {
                MessageBox.Show("항목이 제출되었습니다.");

                this.stackPanel.BindingGroup.BeginEdit();
            }
        }

        #endregion
        #region 취소 버튼 클릭시 처리하기 - cancelButton_Click(sender, e)

        /// <summary>
        /// 취소 버튼 클릭시 처리하기
        /// </summary>
        /// <param name="sender">이벤트 발생자</param>
        /// <param name="e">이벤트 인자</param>
        private void cancelButton_Click(object sender, RoutedEventArgs e)
        {
            this.stackPanel.BindingGroup.CancelEdit();

            this.stackPanel.BindingGroup.BeginEdit();
        }

        #endregion
    }
}