﻿using System;
using System.ComponentModel;

namespace TestProject
{
    /// <summary>
    /// 구매 항목
    /// </summary>
    public class PurchaseItem : INotifyPropertyChanged, IEditableObject
    {
        //////////////////////////////////////////////////////////////////////////////////////////////////// Structure
        ////////////////////////////////////////////////////////////////////////////////////////// Private

        #region 항목 데이터 - ItemData

        /// <summary>
        /// 항목 데이터
        /// </summary>
        private struct ItemData
        {
            //////////////////////////////////////////////////////////////////////////////////////////////////// Field
            ////////////////////////////////////////////////////////////////////////////////////////// Internal

            #region Field

            /// <summary>
            /// 설명
            /// </summary>
            internal string Description;

            /// <summary>
            /// 가격
            /// </summary>
            internal double Price;

            /// <summary>
            /// 제안 만기일
            /// </summary>
            internal DateTime OfferExpireDate;

            #endregion

            //////////////////////////////////////////////////////////////////////////////////////////////////// Method
            ////////////////////////////////////////////////////////////////////////////////////////// Static
            //////////////////////////////////////////////////////////////////////////////// Internal

            #region 신규 항목 구하기 - GetNewItem()

            /// <summary>
            /// 신규 항목 구하기
            /// </summary>
            /// <returns>신규 항목</returns>
            internal static ItemData GetNewItem()
            {
                ItemData data = new ItemData();

                data.Description     = "New item";
                data.Price           = 0;
                data.OfferExpireDate = DateTime.Now + new TimeSpan(7, 0, 0, 0);

                return data;
            }

            #endregion
        }

        #endregion

        //////////////////////////////////////////////////////////////////////////////////////////////////// Event
        ////////////////////////////////////////////////////////////////////////////////////////// Public

        #region 속성 변경시 이벤트 - PropertyChanged

        /// <summary>
        /// 속성 변경시 이벤트
        /// </summary>
        public event PropertyChangedEventHandler PropertyChanged;

        #endregion

        //////////////////////////////////////////////////////////////////////////////////////////////////// Field
        ////////////////////////////////////////////////////////////////////////////////////////// Private

        #region Field

        /// <summary>
        /// 복사 데이터
        /// </summary>
        private ItemData copyData = ItemData.GetNewItem();

        /// <summary>
        /// 현대 데이터
        /// </summary>
        private ItemData currentData = ItemData.GetNewItem();

        #endregion

        //////////////////////////////////////////////////////////////////////////////////////////////////// Property
        ////////////////////////////////////////////////////////////////////////////////////////// Public

        #region 설명 - Description

        /// <summary>
        /// 설명
        /// </summary>
        public string Description
        {
            get
            {
                return this.currentData.Description;
            }
            set
            {
                if(this.currentData.Description != value)
                {
                    this.currentData.Description = value;

                    FirePropertyChangedEvent("Description");
                }
            }
        }

        #endregion
        #region 가격 - Price

        /// <summary>
        /// 가격
        /// </summary>
        public double Price
        {
            get
            {
                return this.currentData.Price;
            }
            set
            {
                if(this.currentData.Price != value)
                {
                    this.currentData.Price = value;

                    FirePropertyChangedEvent("Price");
                }
            }
        }

        #endregion
        #region 제안 만기일 - OfferExpireDate

        /// <summary>
        /// 제안 만기일
        /// </summary>
        public DateTime OfferExpireDate
        {
            get
            {
                return this.currentData.OfferExpireDate;
            }
            set
            {
                if(value != this.currentData.OfferExpireDate)
                {
                    this.currentData.OfferExpireDate = value;

                    FirePropertyChangedEvent("OfferExpires");
                }
            }
        }

        #endregion

        //////////////////////////////////////////////////////////////////////////////////////////////////// Constructor
        ////////////////////////////////////////////////////////////////////////////////////////// Public

        #region 생성자 - PurchaseItem()

        /// <summary>
        /// 생성자
        /// </summary>
        public PurchaseItem()
        {
        }

        #endregion
        #region 생성자 - PurchaseItem(description, price, offerExpireData)

        /// <summary>
        /// 생성자
        /// </summary>
        /// <param name="description">설명</param>
        /// <param name="price">가격</param>
        /// <param name="offerExpireData">제안 만기일</param>
        public PurchaseItem(string description, double price, DateTime offerExpireData)
        {
            Description     = description;
            Price           = price;
            OfferExpireDate = offerExpireData;
        }

        #endregion

        //////////////////////////////////////////////////////////////////////////////////////////////////// Method
        ////////////////////////////////////////////////////////////////////////////////////////// Public

        #region 문자열 구하기 - ToString()

        /// <summary>
        /// 문자열 구하기
        /// </summary>
        /// <returns>문자열</returns>
        public override string ToString()
        {
            return string.Format("{0}, {1:c}, {2:D}", Description, Price, OfferExpireDate);
        }

        #endregion

        #region 편집 시작하기 - BeginEdit()

        /// <summary>
        /// 편집 시작하기
        /// </summary>
        public void BeginEdit()
        {
            this.copyData = this.currentData;
        }

        #endregion
        #region 편집 취소하기 - CancelEdit()

        /// <summary>
        /// 편집 취소하기
        /// </summary>
        public void CancelEdit()
        {
            this.currentData = this.copyData;

            FirePropertyChangedEvent(string.Empty);
        }

        #endregion
        #region 편집 종료하기 - EndEdit()

        /// <summary>
        /// 편집 종료하기
        /// </summary>
        public void EndEdit()
        {
            this.copyData = ItemData.GetNewItem();
        }

        #endregion

        ////////////////////////////////////////////////////////////////////////////////////////// Protected

        #region 속성 변경시 이벤트 발생시키기 - FirePropertyChangedEvent(propertyName)

        /// <summary>
        /// 속성 변경시 이벤트 발생시키기
        /// </summary>
        /// <param name="propertyName">속성명</param>
        private void FirePropertyChangedEvent(string propertyName)
        {
            PropertyChanged?.Invoke(this, new PropertyChangedEventArgs(propertyName));
        }

        #endregion
    }
}